----------------------------------------------------------------------
-- file:   cws_schema.sql
-- action: create CWS schema objects
-- usage:  connect as SYSTEM
--         run this script
--
----------------------------------------------------------------------

set echo on
set define off

spool cws_schema.log

alter session set CURRENT_SCHEMA = CWS_DATA;




create table ANALYSIS_BATCHES
 ( BATCH_ID               varchar2(50)   not null
 , LOCATION_ID            varchar2(50)   not null
 , CONFIGURATION_ID       varchar2(50)   not null
 , START_DATE             date           not null
 , END_DATE               date           not null
 , POLLING_RATE           varchar2(4)
 );

comment on table ANALYSIS_BATCHES is 'batch data to submit for analysis';
comment on column ANALYSIS_BATCHES.BATCH_ID is 'text that names the batch for evaluation';
comment on column ANALYSIS_BATCHES.LOCATION_ID is 'text that names a location';
comment on column ANALYSIS_BATCHES.CONFIGURATION_ID is 'text that names the configuration';
comment on column ANALYSIS_BATCHES.START_DATE is 'the baseline start date';
comment on column ANALYSIS_BATCHES.END_DATE is 'the baseline end date';
comment on column ANALYSIS_BATCHES.POLLING_RATE is 'the time step interval for the batch';

alter table ANALYSIS_BATCHES
 add constraint PK_ABA primary key (BATCH_ID);




create table ANALYSIS_RESULTS
 ( APPLICATION_ID         varchar2(50)   not null
 , LOCATION_ID            varchar2(50)   not null
 , TIME_STEP              date           not null
 , ANALYSIS_COMMENTS      varchar2(1000)
 , DETECTION_INDICATOR    varchar2(10)
 , DETECTION_PROBABILITY  varchar2(50)
 );

comment on table ANALYSIS_RESULTS is 'water quality analyses';
comment on column ANALYSIS_RESULTS.APPLICATION_ID is 'text that names the application posting the results';
comment on column ANALYSIS_RESULTS.LOCATION_ID is 'text that names a location';
comment on column ANALYSIS_RESULTS.TIME_STEP is 'a time value associating samples with results';
comment on column ANALYSIS_RESULTS.ANALYSIS_COMMENTS is 'the text of analysis commentary';
comment on column ANALYSIS_RESULTS.DETECTION_INDICATOR is 'an indicator of overall detection for the time step';
comment on column ANALYSIS_RESULTS.DETECTION_PROBABILITY is 'the probability of overall detection for the time step';

alter table ANALYSIS_RESULTS
 add constraint PK_ARE primary key (APPLICATION_ID, LOCATION_ID, TIME_STEP);




create table ANALYSIS_RESULTS_ARCHIVE
 ( BATCH_ID               varchar2(50)   not null
 , RUN_ID                 varchar2(50)   null
 , APPLICATION_ID         varchar2(50)   not null
 , LOCATION_ID            varchar2(50)   not null
 , TIME_STEP              date           not null
 , ANALYSIS_COMMENTS      varchar2(1000)
 , DETECTION_INDICATOR    varchar2(10)
 , DETECTION_PROBABILITY  varchar2(50)
 , EVENT_STATUS           varchar2(10)
 , RUN_DATE               date
 , RELATED_PARAMETER      varchar2(4000)
 );

comment on table ANALYSIS_RESULTS_ARCHIVE is 'archived water quality analyses';
comment on column ANALYSIS_RESULTS_ARCHIVE.BATCH_ID is 'text that names the batch for evaluation';
comment on column ANALYSIS_RESULTS_ARCHIVE.RUN_ID is 'text that names a Run';
comment on column ANALYSIS_RESULTS_ARCHIVE.APPLICATION_ID is 'text that names the application posting the results';
comment on column ANALYSIS_RESULTS_ARCHIVE.LOCATION_ID is 'text that names a location';
comment on column ANALYSIS_RESULTS_ARCHIVE.TIME_STEP is 'a time value associating samples with results';
comment on column ANALYSIS_RESULTS_ARCHIVE.ANALYSIS_COMMENTS is 'the text of analysis commentary';
comment on column ANALYSIS_RESULTS_ARCHIVE.DETECTION_INDICATOR is 'an indicator of overall detection for the time step';
comment on column ANALYSIS_RESULTS_ARCHIVE.DETECTION_PROBABILITY is 'the probability of overall detection for the time step';
comment on column ANALYSIS_RESULTS_ARCHIVE.EVENT_STATUS is 'the indicator for an event';
comment on column ANALYSIS_RESULTS_ARCHIVE.RUN_DATE is 'time when run executed';

alter table ANALYSIS_RESULTS_ARCHIVE
 add constraint PK_ARC primary key (RUN_ID, TIME_STEP);





create table ANALYSIS_RUNS
 ( BATCH_ID                       varchar2(50)   not null
 , RUN_ID                         varchar2(50)   not null
 , RUN_DESCRIPTION                varchar2(100)  not null
 , EVENT_START_TIME_STEP          date           null
 , CONTAMINANT_ID                 varchar2(50)   null
 , CONTAMINANT_CONCENTRATION      varchar2(10)   null
 , PATTERN_ID                     varchar2(10)   null
 , PEAK_EXTENSION                 varchar2(10)   null
 );

comment on table ANALYSIS_RUNS is 'runs assigned to a batch to submit for analysis';
comment on column ANALYSIS_RUNS.BATCH_ID is 'text that names the batch for evaluation';
comment on column ANALYSIS_RUNS.RUN_ID is 'text that names a run';
comment on column ANALYSIS_RUNS.RUN_DESCRIPTION is 'text that describes the run';
comment on column ANALYSIS_RUNS.EVENT_START_TIME_STEP is 'a time value when events start';
comment on column ANALYSIS_RUNS.CONTAMINANT_ID is 'text that names the contaminant';
comment on column ANALYSIS_RUNS.CONTAMINANT_CONCENTRATION is 'text that names the concentration';
comment on column ANALYSIS_RUNS.PATTERN_ID is 'text that names the pattern';
comment on column ANALYSIS_RUNS.PEAK_EXTENSION is '';

alter table ANALYSIS_RUNS
 add constraint PK_RUN primary key (BATCH_ID, RUN_ID);




create table ANALYSIS_SAMPLES
 ( TIME_STEP        date         not null
 , PARAMETER_ID     varchar2(50) not null
 , SAMPLE_VALUE     varchar2(20)
 , SAMPLE_QUALITY   varchar2(10)
 , EVENT_STATUS     varchar2(10)
 , RECEIPT_TIME     date
 );

comment on table ANALYSIS_SAMPLES is 'parameter data for analysis';
comment on column ANALYSIS_SAMPLES.TIME_STEP is 'a time value associating samples with results';
comment on column ANALYSIS_SAMPLES.PARAMETER_ID is 'a code that identifies a parameter';
comment on column ANALYSIS_SAMPLES.SAMPLE_VALUE is 'the value of a parameter at a location and time';
comment on column ANALYSIS_SAMPLES.SAMPLE_QUALITY is 'the quality of a parameter at a location and time';
comment on column ANALYSIS_SAMPLES.EVENT_STATUS is 'the indicator for an event';
comment on column ANALYSIS_SAMPLES.RECEIPT_TIME is 'the time data was received';

alter table ANALYSIS_SAMPLES
 add constraint PK_ASA primary key (TIME_STEP, PARAMETER_ID);

create index IDX_ES on ANALYSIS_SAMPLES (EVENT_STATUS);




create table ANALYSIS_VARIABLES
 ( APPLICATION_ID           varchar2(50)   not null
 , VARIABLE_NAME            varchar2(100)  not null
 , VARIABLE_DESCRIPTION     varchar2(100)
 , VARIABLE_ACCEPTED_VALUE  varchar2(100)
 , VARIABLE_DEFAULT_VALUE   varchar2(100)
 );

comment on table ANALYSIS_VARIABLES is 'configurable analysis settings';
comment on column ANALYSIS_VARIABLES.APPLICATION_ID is 'a code that identifies an application';
comment on column ANALYSIS_VARIABLES.VARIABLE_NAME is 'text that names an analysis variable';
comment on column ANALYSIS_VARIABLES.VARIABLE_DESCRIPTION is 'text that describes analysis variable';
comment on column ANALYSIS_VARIABLES.VARIABLE_ACCEPTED_VALUE is 'the accepted text value of an analysis variable';
comment on column ANALYSIS_VARIABLES.VARIABLE_DEFAULT_VALUE is 'the default text value of an analysis variable';

alter table ANALYSIS_VARIABLES
 add constraint PK_AVA primary key (APPLICATION_ID, VARIABLE_NAME);




create table APPLICATION_PROPERTIES
 ( APPLICATION_ID  varchar2(50)   not null
 , PROPERTY_NAME   varchar2(100)  not null
 , PROPERTY_VALUE  varchar2(1000)
 );

comment on table APPLICATION_PROPERTIES is 'named characteristics of applications';
comment on column APPLICATION_PROPERTIES.APPLICATION_ID is 'a code that identifies an application';
comment on column APPLICATION_PROPERTIES.PROPERTY_NAME is 'the text name of a property';
comment on column APPLICATION_PROPERTIES.PROPERTY_VALUE is 'the text value of a property';

alter table APPLICATION_PROPERTIES
 add constraint PK_APR primary key (APPLICATION_ID, PROPERTY_NAME);




create table APPLICATION_VARIABLES
 ( CONFIGURATION_ID     varchar2(50)  not null
 , APPLICATION_ID       varchar2(50)  not null
 , VARIABLE_NAME        varchar2(100) not null
 , VARIABLE_VALUE       varchar2(200)
 );

comment on table APPLICATION_VARIABLES is 'application settings configured by application for any location';
comment on column APPLICATION_VARIABLES.CONFIGURATION_ID is 'a code that identifies a configuration';
comment on column APPLICATION_VARIABLES.APPLICATION_ID is 'a code that identifies an application';
comment on column APPLICATION_VARIABLES.VARIABLE_NAME is 'text that names an analysis variable';
comment on column APPLICATION_VARIABLES.VARIABLE_VALUE is 'the text value of an analysis variable set for a location';

alter table APPLICATION_VARIABLES
 add constraint PK_LVA primary key (CONFIGURATION_ID, APPLICATION_ID, VARIABLE_NAME);




create table APPLICATIONS
 ( APPLICATION_ID    varchar2(50)  not null
 , APPLICATION_NAME  varchar2(100)
 , constraint AK_APP unique (APPLICATION_NAME)
 );

comment on table APPLICATIONS is 'software components of the system';
comment on column APPLICATIONS.APPLICATION_ID is 'a code that identifies an application';
comment on column APPLICATIONS.APPLICATION_NAME is 'text that names an application';

alter table APPLICATIONS
 add constraint PK_APP primary key (APPLICATION_ID);




create table BASELINE_DATA
 ( PARAMETER_ID     varchar2(50)    not null
 , TIME_STEP        date            not null
 , SAMPLE_VALUE     varchar2(20)
 , SAMPLE_QUALITY   varchar2(10)
 , EVENT_STATUS     varchar2(10)
);

comment on table BASELINE_DATA is 'baseline data';
comment on column BASELINE_DATA.PARAMETER_ID is 'a code that identifies a contaminant';
comment on column BASELINE_DATA.TIME_STEP is 'a time value associated with a sample';
comment on column BASELINE_DATA.SAMPLE_VALUE is 'the value of a parameter at a location and time';
comment on column BASELINE_DATA.SAMPLE_QUALITY is 'the quality of a parameter at a location and time';
comment on column BASELINE_DATA.EVENT_STATUS is 'identifies a event value';

alter table BASELINE_DATA
 add constraint PK_BAS primary key (PARAMETER_ID, TIME_STEP);




create table CONTAMINANTS
 ( CONTAMINANT_ID    varchar2(50)    not null
 , PARAMETER_TYPE_ID varchar2(100)   not null
 , REACTION_FACTOR   varchar2(50)    not null
 );

comment on table CONTAMINANTS is 'contaminant library';
comment on column CONTAMINANTS.CONTAMINANT_ID is 'a code that identifies a contaminant';
comment on column CONTAMINANTS.PARAMETER_TYPE_ID is 'text that names the type of parameter';
comment on column CONTAMINANTS.REACTION_FACTOR is 'reaction factor value';

alter table CONTAMINANTS
 add constraint PK_CON primary key (CONTAMINANT_ID, PARAMETER_TYPE_ID);




create table DETECTION_INDICATORS
 ( INDICATOR_VALUE        varchar2(10)  not null
 , INDICATOR_DESCRIPTION  varchar2(100)
 );

comment on table DETECTION_INDICATORS is 'codes that indicate detection status';
comment on column DETECTION_INDICATORS.INDICATOR_VALUE is 'the text value of an indicator';
comment on column DETECTION_INDICATORS.INDICATOR_DESCRIPTION is 'text that describes an indicator value';

alter table DETECTION_INDICATORS
 add constraint PK_DET primary key (INDICATOR_VALUE);




create table LOCATION_PARAMETERS
 ( LOCATION_ID   varchar2(50) not null
 , PARAMETER_ID  varchar2(50) not null
 );

comment on table LOCATION_PARAMETERS is 'all sensors by location';
comment on column LOCATION_PARAMETERS.LOCATION_ID is 'a code that identifies a location';
comment on column LOCATION_PARAMETERS.PARAMETER_ID is 'a code that identifies a parameter';

alter table LOCATION_PARAMETERS
 add constraint PK_LPA primary key (LOCATION_ID, PARAMETER_ID);




create table LOCATIONS
 ( LOCATION_ID            varchar2(50)  not null
 , LOCATION_NAME          varchar2(100)
 , LOCATION_DISPLAY_ORDER number(3)
 , constraint AK_LOC unique (LOCATION_NAME)
 );

comment on table LOCATIONS is 'water system facilities';
comment on column LOCATIONS.LOCATION_ID is 'a code that identifies a location';
comment on column LOCATIONS.LOCATION_NAME is 'text that names a location';
comment on column LOCATIONS.LOCATION_DISPLAY_ORDER is 'number indicating sort order of location';

alter table LOCATIONS
 add constraint PK_LOC primary key (LOCATION_ID);




create table PARAMETER_TYPE_RESULTS
 ( APPLICATION_ID       varchar2(50)  not null
 , LOCATION_ID          varchar2(50)  not null
 , TIME_STEP            date          not null
 , PARAMETER_TYPE_ID    varchar2(100) not null
 );

comment on table PARAMETER_TYPE_RESULTS is 'analyses of water quality attributes';
comment on column PARAMETER_TYPE_RESULTS.APPLICATION_ID is 'a code that identifies an application';
comment on column PARAMETER_TYPE_RESULTS.LOCATION_ID is 'text that names a location';
comment on column PARAMETER_TYPE_RESULTS.TIME_STEP is 'a time value associating samples with results';
comment on column PARAMETER_TYPE_RESULTS.PARAMETER_TYPE_ID is 'text that names a parameter type';

alter table PARAMETER_TYPE_RESULTS
 add constraint PK_PTR primary key (APPLICATION_ID, LOCATION_ID, TIME_STEP, PARAMETER_TYPE_ID);




create table PARAMETER_TYPE_RESULTS_ARCHIVE
 ( BATCH_ID             varchar2(50)  not null
 , RUN_ID               varchar2(50)  not null
 , APPLICATION_ID       varchar2(50)  not null
 , LOCATION_ID          varchar2(50)  not null
 , TIME_STEP            date          not null
 , PARAMETER_TYPE_ID    varchar2(100) not null
 );

comment on table PARAMETER_TYPE_RESULTS_ARCHIVE is 'archive of analyses of water quality attributes';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.BATCH_ID is 'text that names the batch for evaluation';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.RUN_ID is 'text that names a Run';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.APPLICATION_ID is 'a code that identifies an application';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.LOCATION_ID is 'text that names a location';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.TIME_STEP is 'a time value associating samples with results';
comment on column PARAMETER_TYPE_RESULTS_ARCHIVE.PARAMETER_TYPE_ID is 'text that names a parameter type';

alter table PARAMETER_TYPE_RESULTS_ARCHIVE
 add constraint PK_PTRA primary key (RUN_ID, APPLICATION_ID, LOCATION_ID, TIME_STEP, PARAMETER_TYPE_ID);




create table PARAMETER_TYPES
 ( PARAMETER_TYPE_ID           varchar2(100)  not null
 , PARAMETER_TYPE_NAME         varchar2(100)
 , MINIMUM_VALUE               NUMBER(5,2)
 , MAXIMUM_VALUE	       NUMBER(8,2)
 , constraint AK_PTY unique (PARAMETER_TYPE_NAME)
 );

comment on table PARAMETER_TYPES is 'attributes of water quality';
comment on column PARAMETER_TYPES.PARAMETER_TYPE_ID is 'a code that identifies a parameter type';
comment on column PARAMETER_TYPES.PARAMETER_TYPE_NAME is 'text that names a parameter type';
comment on column PARAMETER_TYPES.MINIMUM_VALUE is 'number that identifies what the minimum value should be';
comment on column PARAMETER_TYPES.MAXIMUM_VALUE is 'number that identifies what the maximum value should be';

alter table PARAMETER_TYPES
 add constraint PK_PTY primary key (PARAMETER_TYPE_ID);




create table PARAMETERS
 ( PARAMETER_ID           varchar2(50)  not null
 , PARAMETER_TYPE_ID      varchar2(100)  not null
 , PARAMETER_NAME         varchar2(50)
 , SETPOINT_LO            NUMBER(5,2)
 , SETPOINT_LOLO          NUMBER(5,2)
 , SETPOINT_HI            NUMBER(5,2)
 , SETPOINT_HIHI          NUMBER(5,2)
 , constraint AK_PAR unique (PARAMETER_NAME)
 );

comment on table PARAMETERS is 'water quality sensors';
comment on column PARAMETERS.PARAMETER_ID is 'a code that identifies a parameter';
comment on column PARAMETERS.PARAMETER_TYPE_ID is 'a code that identifies a parameter type';
comment on column PARAMETERS.PARAMETER_NAME is 'text that names a parameter';

alter table PARAMETERS
 add constraint PK_PAR primary key (PARAMETER_ID);




create table PATTERNS
 ( PATTERN_ID                     varchar2(50)  not null
 , PERCENT_PEAK_CONCENTRATION     varchar2(20)  not null
 , STEPS                          varchar2(10)
 );

comment on table PATTERNS is 'pattern library';
comment on column PATTERNS.PATTERN_ID is 'a code that identifies a pattern';
comment on column PATTERNS.PERCENT_PEAK_CONCENTRATION is 'peak concentration percentage';
comment on column PATTERNS.STEPS is 'number of steps';

alter table PATTERNS
 add constraint PK_PAT primary key (PATTERN_ID, STEPS);



alter table ANALYSIS_BATCHES
       add constraint FK_LOC_ABA foreign key (LOCATION_ID) references LOCATIONS (LOCATION_ID);
alter table ANALYSIS_RESULTS
       add constraint FK_APP_ARE foreign key (APPLICATION_ID) references APPLICATIONS (APPLICATION_ID);
alter table ANALYSIS_RESULTS
       add constraint FK_LOC_ARE foreign key (LOCATION_ID) references LOCATIONS (LOCATION_ID);
alter table ANALYSIS_RESULTS_ARCHIVE
       add constraint FK_BAT_ARC foreign key (BATCH_ID, RUN_ID) references ANALYSIS_RUNS (BATCH_ID, RUN_ID);
alter table ANALYSIS_RESULTS_ARCHIVE
       add constraint FK_APP_ARC foreign key (APPLICATION_ID) references APPLICATIONS (APPLICATION_ID);
alter table ANALYSIS_RESULTS_ARCHIVE
       add constraint FK_LOC_ARC foreign key (LOCATION_ID) references LOCATIONS (LOCATION_ID);
alter table ANALYSIS_RUNS
       add constraint FK_BAT_RUN foreign key (BATCH_ID) references ANALYSIS_BATCHES (BATCH_ID);
alter table ANALYSIS_SAMPLES
       add constraint FK_PAR_ASA foreign key (PARAMETER_ID) references PARAMETERS;
alter table ANALYSIS_VARIABLES
       add constraint FK_APP_AVA foreign key (APPLICATION_ID) references APPLICATIONS;
alter table APPLICATION_PROPERTIES
       add constraint FK_APP_APR foreign key (APPLICATION_ID) references APPLICATIONS;
alter table APPLICATION_VARIABLES
       add constraint FK_APP_APA foreign key (APPLICATION_ID) references APPLICATIONS;
alter table APPLICATION_VARIABLES
       add constraint FK_AVA_APA foreign key (APPLICATION_ID, VARIABLE_NAME) references ANALYSIS_VARIABLES;
alter table BASELINE_DATA
       add constraint FK_PAR_BAS foreign key (PARAMETER_ID) references PARAMETERS;
alter table CONTAMINANTS
       add constraint FK_PTY_CON foreign key (PARAMETER_TYPE_ID) references PARAMETER_TYPES;
alter table LOCATION_PARAMETERS
       add constraint FK_LOC_LPA foreign key (LOCATION_ID) references LOCATIONS;
alter table LOCATION_PARAMETERS
       add constraint FK_PAR_LPA foreign key (PARAMETER_ID) references PARAMETERS;
alter table PARAMETER_TYPE_RESULTS
       add constraint FK_ALT_PTR foreign key (APPLICATION_ID, LOCATION_ID, TIME_STEP) references ANALYSIS_RESULTS;
alter table PARAMETER_TYPE_RESULTS
       add constraint FK_PTY_PTR foreign key (PARAMETER_TYPE_ID) references PARAMETER_TYPES (PARAMETER_TYPE_ID);
alter table PARAMETER_TYPE_RESULTS
       add constraint FK_APP_PTR foreign key (APPLICATION_ID) references APPLICATIONS (APPLICATION_ID);
alter table PARAMETERS
       add constraint FK_PTY_PAR foreign key (PARAMETER_TYPE_ID) references PARAMETER_TYPES;

create or replace
view ANALYSIS_SAMPLES_EDS
 ( LOCATION_ID
 , TIME_STEP
 , PARAMETER_ID
 , PARAMETER_TYPE_ID
 , SAMPLE_VALUE
 , SAMPLE_QUALITY
 ) as
 select
   LOCATION_ID
 , TIME_STEP
 , PARAMETER_ID
 , PARAMETER_TYPE_ID
 , SAMPLE_VALUE
 , SAMPLE_QUALITY
 from ANALYSIS_SAMPLES
 join LOCATION_PARAMETERS using (PARAMETER_ID)
 join PARAMETERS using (PARAMETER_ID)
 join PARAMETER_TYPES using (PARAMETER_TYPE_ID)
;

create or replace
view APPLICATION_VARIABLES_EDS
 ( CONFIGURATION_ID
 , APPLICATION_ID
 , VARIABLE_NAME
 , VARIABLE_VALUE
 ) as
 select
   CONFIGURATION_ID
 , APPLICATION_ID
 , VARIABLE_NAME
 , VARIABLE_VALUE
 from APPLICATION_VARIABLES
;



spool off
